searchData={"content_type":"text/markdown","items":[{"doc":"Uinta is a plugin for the default Elixir logger that lowers log volume while\nmaximizing log usefulness. It is not a logger backend, but rather includes\n`Uinta.Formatter` which will format logs on top of the default Elixir logger\nbackend.\n\nIn addition to the formatter, Uinta also includes `Uinta.Plug`. The plug is a\ndrop-in replacement for `Plug.Logger` that will log out the request and\nresponse on a single line. It can also put the request info into the\ntop-level JSON for easier parsing by your log aggregator.","ref":"Uinta.html","title":"Uinta","type":"module"},{"doc":"The formatter and plug will be installed separately depending on the\nfunctionality that you want.\n\nTo install the formatter, see the instructions in `Uinta.Formatter`\n\nTo install the plug, see the instructions in `Uinta.Plug`","ref":"Uinta.html#module-installation","title":"Installation - Uinta","type":"module"},{"doc":"The Uinta Formatter will wrap normal log statements in a JSON object. The log\nlevel, timestamp, and metadata will all be attached to it as parts of the\nobject.\n\nThe formatter can also format stuctured log messages. When a JSON string is\nreceived for formatting, it will be decoded and merged with the map to be\noutput. In this way any keys that are passed to it will be on the high level\nobject, so that they won't need to be extracted from a secondary object later\non.\n\nJSON tends to be a great solution for making logs easily machine parseable,\nwhile still being mostly human readable. However, it is recommended that if\nyou have separate configuration for development and production environments\nthat you only enable this in the production environment as it can still\ndecrease developer productivity to have to mentally parse JSON during\ndevelopment.","ref":"Uinta.Formatter.html","title":"Uinta.Formatter","type":"module"},{"doc":"To use the formatter, you'll need to add it to your logger configuration. In\nyour (production) config file, see if you have a line that looks something\nlike this:\n\n```\nconfig :logger, :console, format: \"[$level] $message\\n\"\n```\n\nIf you have it, you'll want to replace it with this:\n\n```\nconfig :logger, :console, format: {Uinta.Formatter, :format}\n```\n\nIf you don't have it, you'll want to just add that line.","ref":"Uinta.Formatter.html#module-installation","title":"Installation - Uinta.Formatter","type":"module"},{"doc":"#","ref":"Uinta.Formatter.html#module-available-formatters","title":"Available Formatters - Uinta.Formatter","type":"module"},{"doc":"No special handling, outputs everything as JSON.\n\n#","ref":"Uinta.Formatter.html#module-standard","title":"Standard - Uinta.Formatter","type":"module"},{"doc":"Support adding Datadog specific metadata to logs.","ref":"Uinta.Formatter.html#module-datadog","title":"Datadog - Uinta.Formatter","type":"module"},{"doc":"This function takes in four arguments, as defined by\n[Logger](https://hexdocs.pm/logger/Logger.html#module-custom-formatting):\n\n  - `level` is the log level, one of `:debug`, `:info`, `:warn`, and `:error`\n  - `message` is the message to be formatted. This should be iodata\n(typically String or iolist)\n  - `timestamp` is a timestamp formatted according to\n`t:Logger.Formatter.time/0`\n  - `metadata` is a keyword list containing metadata that will be included\nwith the log line\n\nHowever, this line should not be called manually. Instead it should be called\nby configuring the Elixir logger in your project to use it as a custom log\nformatter. See [the installation instructions](#module-installation) for more\ninformation.\n\nDelegates to Uinta.Formatter.Standard for backward compatibility","ref":"Uinta.Formatter.html#format/4","title":"Uinta.Formatter.format/4","type":"function"},{"doc":"","ref":"Uinta.Formatter.Datadog.html","title":"Uinta.Formatter.Datadog","type":"module"},{"doc":"```\nconfig :logger, :console, format: {Uinta.Formatter.Datadog, :format}\n```","ref":"Uinta.Formatter.Datadog.html#module-installation","title":"Installation - Uinta.Formatter.Datadog","type":"module"},{"doc":"#","ref":"Uinta.Formatter.Datadog.html#module-features","title":"Features - Uinta.Formatter.Datadog","type":"module"},{"doc":"Takes trace ids and span ids in the OpenTelemetry format (as hex) and convert them into Datadog\nformat in the correct key. Logger.metadata must include the keys `:trace_id` and `:span_id`.\nBe sure to add those two keys onto the allowlist in the `:logger` config. See\n[Datadog's documentation](https://docs.datadoghq.com/tracing/connect_logs_and_traces/opentelemetry/)\nfor more details.\n\n##","ref":"Uinta.Formatter.Datadog.html#module-trace-and-span-id-correlation","title":"Trace and Span id correlation - Uinta.Formatter.Datadog","type":"module"},{"doc":"```elixir\nctx = OpenTelemetry.Tracer.current_span_ctx()\nLogger.metadata([\n  trace_id: OpenTelemetry.Span.hex_trace_id(ctx),\n  span_id: OpenTelemetry.Span.hex_span_id(ctx)\n])\n```","ref":"Uinta.Formatter.Datadog.html#module-example","title":"Example - Uinta.Formatter.Datadog","type":"module"},{"doc":"See Uinta.formatter.format/4","ref":"Uinta.Formatter.Datadog.html#format/4","title":"Uinta.Formatter.Datadog.format/4","type":"function"},{"doc":"No special handling, outputs everything as JSON. See Uinta.Formatter for more information.","ref":"Uinta.Formatter.Standard.html","title":"Uinta.Formatter.Standard","type":"module"},{"doc":"See Uinta.formatter.format/4","ref":"Uinta.Formatter.Standard.html#format/4","title":"Uinta.Formatter.Standard.format/4","type":"function"},{"doc":"","ref":"Uinta.Formatter.Standard.html#t:level/0","title":"Uinta.Formatter.Standard.level/0","type":"type"},{"doc":"","ref":"Uinta.Formatter.Standard.html#t:time/0","title":"Uinta.Formatter.Standard.time/0","type":"type"},{"doc":"Utilities for building a formatter","ref":"Uinta.Formatter.Util.html","title":"Uinta.Formatter.Util","type":"module"},{"doc":"Stringify as JSON","ref":"Uinta.Formatter.Util.html#encode/1","title":"Uinta.Formatter.Util.encode/1","type":"function"},{"doc":"Format as a map with metadata as a map, timestamp, level, and message.","ref":"Uinta.Formatter.Util.html#format/4","title":"Uinta.Formatter.Util.format/4","type":"function"},{"doc":"RFC3339 UTC \"Zulu\" format.","ref":"Uinta.Formatter.Util.html#format_timestamp/1","title":"Uinta.Formatter.Util.format_timestamp/1","type":"function"},{"doc":"This plug combines the request and response logs into a single line. This\nbrings many benefits including:\n\n- Removing the need to visually match up the request and response makes it\neasier to read your logs and get a full picture of what has happened.\n\n- Having a single line for both request and response halves the number of\nrequest logs that your log aggregator will need to process and index, which\nleads to saved costs\n\nIn addition to combining the log lines, it also gives you the ability to\noutput request logs in JSON format so that you can easily have your log\naggregator parse the fields. To do this, pass `json: true` in the options\nwhen calling the plug.\n\nYou will also gain the ability to not log certain paths that are requested,\nas long as those paths return a 200-level status code. This can be\nparticularly useful for things like not showing health checks in your logs\nto cut down on noise. To do this, just pass `ignored_paths:\n[\"/path_to_ignore\"]` in the options.\n\nFinally, GraphQL requests will replace `POST /graphql` with the GraphQL\noperation type and name like `QUERY getUser` or `MUTATION createUser` if an\noperation name is provided. This will give you more visibility into your\nGraphQL requests without having to log out the entire request body or go\ninto debug mode. If desired, the GraphQL variables can be included in the\nlog line as well. The query can also be included if unnamed.","ref":"Uinta.Plug.html","title":"Uinta.Plug","type":"module"},{"doc":"Installation of the plug will depend on how your app currently logs requests.\nOpen `YourApp.Endpoint` and look for the following line:\n\n```\nplug Plug.Logger\n```\n\nIf it exists in your endpoint, replace it with this (using the options you\nwant):\n\n```\nplug Uinta.Plug,\n  log: :info,\n  format: :string,\n  include_variables: false,\n  ignored_paths: [],\n  filter_variables: [],\n  success_log_sampling_ratio: 1.0,\n  include_datadog_fields: false\n```\n\nIf your endpoint didn't call `Plug.Logger`, add the above line above the line\nthat looks like this:\n\n```\nplug Plug.RequestId\n```\n\nNow you will also want to add the following anywhere in your main config file to\nmake sure that you aren't logging each request twice:\n\n```\nconfig :phoenix, logger: false\n```","ref":"Uinta.Plug.html#module-installation","title":"Installation - Uinta.Plug","type":"module"},{"doc":"- `:log` - The log level at which this plug should log its request info.\nDefault is `:info`\n- `:format` - Output format, either :json, :string, or :map. Default is `:string`\n- `:json` - Whether or not plug should log in JSON format. Default is `false` (obsolete)\n- `:ignored_paths` - A list of paths that should not log requests. Default\nis `[]`.\n- `:include_variables` - Whether or not to include any GraphQL variables in\nthe log line when applicable. Default is `false`.\n- `:filter_variables` - A list of variable names that should be filtered\nout from the logs. By default `password`, `passwordConfirmation`,\n`idToken`, and `refreshToken` will be filtered.\n- `:include_unnamed_queries` - Whether or not to include the full query\nbody for queries with no name supplied\n- `:success_log_sampling_ratio` - What percentage of successful requests\nshould be logged. Defaults to 1.0\n- `:include_datadog_fields` - Whether or not to add logger specific field based on Datadog logger.  Default is\n`false`. See https://docs.datadoghq.com/logs/log_configuration/attributes_naming_convention/#http-requests for details","ref":"Uinta.Plug.html#module-options","title":"Options - Uinta.Plug","type":"module"},{"doc":"","ref":"Uinta.Plug.html#client_ip/1","title":"Uinta.Plug.client_ip/1","type":"function"},{"doc":"","ref":"Uinta.Plug.html#t:format/0","title":"Uinta.Plug.format/0","type":"type"},{"doc":"","ref":"Uinta.Plug.html#t:graphql_info/0","title":"Uinta.Plug.graphql_info/0","type":"type"},{"doc":"","ref":"Uinta.Plug.html#t:opts/0","title":"Uinta.Plug.opts/0","type":"type"},{"doc":"Defines types","ref":"Uinta.Types.html","title":"Uinta.Types","type":"module"},{"doc":"","ref":"Uinta.Types.html#t:level/0","title":"Uinta.Types.level/0","type":"type"},{"doc":"","ref":"Uinta.Types.html#t:time/0","title":"Uinta.Types.time/0","type":"type"},{"doc":"# CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-13-0-2024-01-09","title":"v0.13.0 (2024-01-09) - CHANGELOG","type":"extras"},{"doc":"* Support not double encoding the payload. In order to do that, a new plugs option `format` was added. We are deprecating the `json` option instead though it is backward compatible for a little while\n\n   - `:format` - Output format, either :json, :string, or :map. Default is `:string`\n\n  To migrate easily, just find where you initialize your plug, and change the :json to :format","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-12-1-2023-12-01","title":"v0.12.1 (2023-12-01) - CHANGELOG","type":"extras"},{"doc":"* handle white space in front of a mutation/query","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-12-0-2023-11-07","title":"v0.12.0 (2023-11-07) - CHANGELOG","type":"extras"},{"doc":"* Upgrade dependencies\n* fix bad regex to capture better operation name","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-11-0-2022-11-28","title":"v0.11.0 (2022-11-28) - CHANGELOG","type":"extras"},{"doc":"* Upgrade dependencies\n * Clean up some docs\n * some internal code cleaning\n * ** Breaking **  Support Elixir 1.11 and above","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-10-4-2022-07-25","title":"v0.10.4 (2022-07-25) - CHANGELOG","type":"extras"},{"doc":"* Add support for formatting metadata list values","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-10-3-2022-07-19","title":"v0.10.3 (2022-07-19) - CHANGELOG","type":"extras"},{"doc":"* Log out error on rescue when we are unable to format","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-10-2-2022-05-31","title":"v0.10.2 (2022-05-31) - CHANGELOG","type":"extras"},{"doc":"* Removed remapping for status as done in Datadog directly","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-10-1-2022-05-31","title":"v0.10.1 (2022-05-31) - CHANGELOG","type":"extras"},{"doc":"* Adds support for mapped fields in DataDog. To enable this, use `include_datadog_fields: true` in your plug initialization","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-10-0-2022-05-09","title":"v0.10.0 (2022-05-09) - CHANGELOG","type":"extras"},{"doc":"* `Uinta.Formatter.Datadog` - Drop in replacement for `Uinta.Formatter` that adds Datadog specific metadata to help correlate traces and logs","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-9-2-2022-03-08","title":"v0.9.2 (2022-03-08) - CHANGELOG","type":"extras"},{"doc":"* Doesn't crash when \"query\" is not a string","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-9-1-2022-01-21","title":"v0.9.1 (2022-01-21) - CHANGELOG","type":"extras"},{"doc":"* Client ip is now properly serialize as a string","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-9-2022-01-05","title":"v0.9 (2022-01-05) - CHANGELOG","type":"extras"},{"doc":"* adds more fields to the log:\n  * referer\n  * user_agent\n  * x_forwarded_for\n  * x_forwarded_proto\n  * x_forwarded_port\n  * via","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-8-2021-11-22","title":"v0.8 (2021-11-22) - CHANGELOG","type":"extras"},{"doc":"`operationName` for GQL requests\n\n#","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"`path` now always path and not sometimes `operationName`","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-7-2021-08-16","title":"v0.7 (2021-08-16) - CHANGELOG","type":"extras"},{"doc":"* configurable sampling of successful requests. Use the :success_log_sampling_ratio configuration in init and specify a ratio of the sample to log. Ratio can support a precision up to 4 digits\n* update dependencies","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-6-2021-04-08","title":"v0.6 (2021-04-08) - CHANGELOG","type":"extras"},{"doc":"* ignore request path options to `Uinta.Plug`, to filter out request unwanted unless HTTP status returned\nis not a 200-level status\n* Adds a CHANGELOG\n* Adds a duration_ms as a number\n\n#","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"* Upgraded the package dependencies","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-5-2020-11-04","title":"v0.5 (2020-11-04) - CHANGELOG","type":"extras"},{"doc":"* ignore request path options to `Uinta.Plug`, to filter out request unwanted unless HTTP status returned\nis not a 200-level status","ref":"changelog.html#added","title":"Added - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-4-2-2020-03-31","title":"v0.4.2 (2020-03-31) - CHANGELOG","type":"extras"},{"doc":"* properly handle GraphQL queries with no commas","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-4-1-2020-03-31","title":"v0.4.1 (2020-03-31) - CHANGELOG","type":"extras"},{"doc":"* handle array arguments properly in the regex","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"#","ref":"changelog.html#v0-4-0-2020-03-30","title":"v0.4.0 (2020-03-30) - CHANGELOG","type":"extras"},{"doc":"* better handle queries with no operationName","ref":"changelog.html#changed","title":"Changed - CHANGELOG","type":"extras"},{"doc":"# Uinta\n\n[![Build Status](https://github.com/podium/uinta/actions/workflows/ci.yml/badge.svg)](https://github.com/podium/uinta/actions/workflows/ci.yml) [![Hex.pm](https://img.shields.io/hexpm/v/uinta.svg)](https://hex.pm/packages/uinta) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/uinta)\n[![Total Download](https://img.shields.io/hexpm/dt/uinta.svg)](https://hex.pm/packages/uinta)\n[![License](https://img.shields.io/hexpm/l/uinta.svg)](https://github.com/podium/uinta/blob/master/LICENSE.md)\n\nUinta is a plugin for the default Elixir logger that lowers log volume while\nmaximizing log usefulness. It is not a logger backend, but rather includes\n`Uinta.Formatter` which will format logs on top of the default Elixir logger\nbackend.\n\nIn addition to the formatter, Uinta also includes `Uinta.Plug`. The plug is a\ndrop-in replacement for `Plug.Logger` that will log out the request and response\non a single line. It can also put the request info into the top-level JSON for\neasier parsing by your log aggregator.","ref":"readme.html","title":"Readme","type":"extras"},{"doc":"At Podium we log millions of lines per minute and store around a terabyte of log\ndata per day. A large percentage of those lines are the typical `GET /` and\n`Sent 200 in 2ms` that `Phoenix.Logger` sends by default. By combining those\ninto a single line, we're able to cut out that percentage of lines so that the\nindexes in our Elasticsearch cluster will be smaller and searches will be\nfaster.\n\nIn addition, about 2/3 of those requests are GraphQL requests. Their first log\nline simply says `POST /graphql` every time, which gives us no insight into what\nthe request is actually doing. `Uinta.Plug` will extract GraphQL query names\nwhen they exist to make these log lines more useful without having to enable\ndebug logs: `QUERY messagesForLocation (/graphql)` or `MUTATION createMessage (/graphql)`.\n\nFor smaller organizations, the ability to filter out lines pertaining to certain\nrequests paths can also be useful to cut down on log noise. Kubernetes health\nchecks and other requests don't usually need to show up in the logs, so\n`Uinta.Plug` allows you to ignore certain paths as long as they return a\n200-level status code.\n\nWhen set up to do so, Uinta will additionally wrap the log line in a JSON object\nso that it can more easily be parsed by Fluentbit and other log parsers. This\nincreases log line size, but improves searchability and makes logs more useful.","ref":"readme.html#why-uinta","title":"Why Uinta? - Readme","type":"extras"},{"doc":"The package can be installed by adding `uinta` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:uinta, \"~> 0.6\"}\n  ]\nend\n```\n\n#","ref":"readme.html#installation","title":"Installation - Readme","type":"extras"},{"doc":"##","ref":"readme.html#available-formatters","title":"Available Formatters - Readme","type":"extras"},{"doc":"No special formatting or fields, good for general use.\n\nModule: `Uinta.Formatter` (or `Uinta.Formatter.Standard`)\n\n##","ref":"readme.html#standard","title":"Standard - Readme","type":"extras"},{"doc":"Adds Datadog specific metadata to the log output. See the module for more setup information and details.\n\nModule: `Uinta.Formatter.Datadog`\n\nTo enable the full log format to use DataDog log service, just use `include_datadog_fields: true` in your plug initialization\n\n#","ref":"readme.html#datadog","title":"Datadog - Readme","type":"extras"},{"doc":"To use the formatter, you'll need to add it to your logger configuration. In\nyour (production) config file, look for a line that looks something like\nthis:\n\n```elixir\nconfig :logger, :console, format: \"[$level] $message\\\\n\"\n```\n\nYou'll want to replace it with this:\n\n```elixir\nconfig :logger, :console, format: {Uinta.Formatter, :format}\n```\n\n#","ref":"readme.html#formatter-installation","title":"Formatter Installation - Readme","type":"extras"},{"doc":"Installation of the plug will depend on how your app currently logs requests.\nOpen `YourApp.Endpoint` and look for the following line:\n\n```elixir\nplug Plug.Logger\n```\n\nIf it exists in your endpoint, replace it with this (using the options you\nwant):\n\n```elixir\nplug Uinta.Plug, format: :string, log: :info\n```\n\nYou can also perform log sampling by setting the `success_log_sampling_ratio`. Following is a 20% log sampling\n\n```elixir\nplug Uinta.Plug, success_log_sampling_ratio: 0.2\n```\n\nIf your endpoint didn't call `Plug.Logger`, add the above line above the line\nthat looks like this:\n\n```elixir\nplug Plug.RequestId\n```\n\nNow you will also want to add the following anywhere in your main config file to\nmake sure that you aren't logging each request twice:\n\n```elixir\nconfig :phoenix, logger: false\n```","ref":"readme.html#plug-installation","title":"Plug Installation - Readme","type":"extras"},{"doc":"Much of this work, especially `Uinta.Plug`, is based on Elixir's\n[`Plug.Logger`](https://github.com/elixir-plug/plug/blob/v1.9.0/lib/plug/logger.ex)","ref":"readme.html#attribution","title":"Attribution - Readme","type":"extras"}]}